# Ferdinand.Ihringer@gmail.com

# Run the following code to generate 350 files in LP format
# for an ILP solver such as Gurobi

# The program expects the files "all_ovoids_of_H_4_1_gap.g"
# and "lines_for_gap.g" in the same folder.

# It writes the 350 ILPs into the folder "ilps/"
# All of this can be changed.

# The function WriteDownIntLP takes a lower and a upper bound on the objects,
# so by changing the lower bound from q^4+q^2+1 to something smaller, one can test 
# for partial distance-2 ovoids

structure_name := "splitcayleyovoid";
dir := "ilps/";

WriteDownIntLP := function(pts, lns, config, size, size2, name)
  local file, output, string, y;
  
  file := Concatenation(structure_name, "_name_", name);
  output := OutputTextFile(Concatenation(dir, file, ".lp"), false);
  
  WriteAll(output, "Maximize\n");
  WriteAll(output, Concatenation(Concatenation(List([1..Size(pts)], x -> Concatenation("PT", String(x), " + "))), " NULL\n"));
  WriteAll(output, "Subject To\n");
  WriteAll(output, Concatenation(List(config, x -> Concatenation("PT", String(x), " >= 1\n"))));
  WriteAll(output, Concatenation(List(config, x -> Concatenation("PT", String(x), " <= 1\n"))));
  WriteAll(output, Concatenation(Concatenation(List([1..Size(pts)], x -> Concatenation("PT", String(x), " + "))), " NULL >= ", String(size2), "\n"));
  WriteAll(output, Concatenation(Concatenation(List([1..Size(pts)], x -> Concatenation("PT", String(x), " + "))), " NULL <= ", String(size), "\n"));
  for y in [1..Size(lns)] do
    WriteAll(output, Concatenation("PL", String(y), ": ", Concatenation(List(Filtered([1..Size(pts)], x -> pts[x] in lns[y]), x -> Concatenation("PT", String(x), " + "))), " NULL <= 1\n"));
  od;
  WriteAll(output, Concatenation(List([1..Size(pts)], x -> Concatenation("PT", String(x), " <= 1\n"))));
  WriteAll(output, "Bounds\n");
  WriteAll(output, Concatenation(List([1..Size(pts)], x -> Concatenation("PT", String(x), " >= 0\n"))));
  WriteAll(output, "NULL <= 0\n");
  WriteAll(output, "Integers\n");
  WriteAll(output, Concatenation(Concatenation(List([1..Size(pts)], x -> Concatenation("PT", String(x), " "))), " "));
  WriteAll(output, "NULL\n");
  WriteAll(output, "End\n");
  
  CloseStream(output);
end;


WriteDownIntLPMinion := function(pts, lns, config, size, size2, name)
  local file, output, string, y;
  
  file := Concatenation(structure_name, "_name_", name);
  output := OutputTextFile(Concatenation(dir, file, ".minion"), false);
  
  WriteAll(output, "MINION 3\n\n");
  WriteAll(output, "**VARIABLES**\n\n");
  WriteAll(output, Concatenation(Concatenation(List([1..Size(pts)], x -> Concatenation("BOOL PT", String(x), "\n"))), ""));
  WriteAll(output, "**CONSTRAINTS**\n");
  WriteAll(output, Concatenation(List(config, x -> Concatenation("eq(PT", String(x), ", 1)\n"))));
  WriteAll(output, Concatenation("sumgeq([", Concatenation(List([1..Size(pts)], x -> Concatenation("PT", String(x), ", "))), "], ", String(size2), ")\n"));
  WriteAll(output, Concatenation("sumleq([", Concatenation(List([1..Size(pts)], x -> Concatenation("PT", String(x), ", "))), "], ", String(size), ")\n"));
  for y in [1..Size(lns)] do
    WriteAll(output, Concatenation("sumleq([", Concatenation(List(Filtered([1..Size(pts)], x -> pts[x] in lns[y]), x -> Concatenation("PT", String(x), ", "))), "], 1)\n"));
    WriteAll(output, Concatenation("sumgeq([", Concatenation(List(Filtered([1..Size(pts)], x -> pts[x] in lns[y]), x -> Concatenation("PT", String(x), ", "))), "], 1)\n"));
  od;

  WriteAll(output, "**EOF**\n");
  
  CloseStream(output);
end;

Read("all_ovoids_of_H_4_1_gap.g");
Read("lines_for_gap.g");

q := 4;
for i in [1..Size(ovoids)] do
  WriteDownIntLP(points, lines, ovoids[i], q^4+q^2+1, q^4+q^2+1, String(i));
  WriteDownIntLPMinion(points, lines, ovoids[i], q^4+q^2+1, q^4+q^2+1, String(i));
  Display(i);
od;


